from django.utils.translation import ugettext_lazy as _

from ..base.models import BaseOAuth2Profile


class PayPalOAuthProfile(BaseOAuth2Profile):
    class Meta:
        verbose_name = _('paypal profile')
        verbose_name_plural = _('paypal profiles')



from datetime import timedelta
from urlparse import urlparse, parse_qs

from django.contrib import messages
from django.contrib.auth.models import User
from django.db.models import Q
from django.http import HttpResponseForbidden
from django.shortcuts import redirect
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.views.generic.base import View, TemplateView

from socialnetworks.base.settings import (
    EMAIL_IS_USERNAME, SETUP_FORM_CLASS, SETUP_TEMPLATE
)
from socialnetworks.signals import connect, disconnect, login


class OAuthMixin(object):
    """
    Mixin that defines the necessary methods for OAuth views.
    OAuth views must be inherited from this class.

    """
    # The client that defines the necessary information to perform
    # the OAuth flow.
    client = None

    def __get_session(self):
        """
        Returns the current session's object to store the data that will
        be used trough the OAuth flow.

        """
        if self.client.session_key not in self.request.session:
            self.request.session[self.client.session_key] = {}

        return self.request.session[self.client.session_key]

    def __set_session(self, dictionary):
        """
        Updates the current session's object, useful if the backend is not
        configured to save session in every request.

        """
        self.request.session[self.client.session_key] = dictionary

    def session_get(self, key):
        """
        Returns the provided key value from the current session's object. If
        key is not in the object returns None.

        """
        return self.__get_session().get(key, None)

    def session_put(self, **dictionary):
        """
        Puts the given dictionary into the current session's object.

        """
        session = self.__get_session()
        session.update(**dictionary)
        self.__set_session(session)

    def session_pop(self, key):
        """
        Returns the provided key value from the current session's object and
        removes the key from the object. If key is not in the object returns
        None.

        """
        session = self.__get_session()
        value = session.pop(key, None)
        self.__set_session(session)

        return value

    def session_clear(self):
        """
        Removes the object where the flow information of the service are
        stored from the current user's session.

        """
        if self.client.session_key in self.request.session:
            del self.request.session[self.client.session_key]

    def get_callback_url(self):
        """
        Returns the url where the user will be redirected after when the
        access token is requested.

        """
        raise NotImplementedError

    def get_redirect_url(self):
        """
        Returns the url where the user will be redirected after its request
        is processed and is successfully logged in.

        """
        raise NotImplementedError

    def get_profile(self):
        """
        Returns the current user's OAuth profile or None.

        """
        raise NotImplementedError

    def create_new_user(self, data):
        """
        Returns a new user, created from the data dictionary.

        """
        raise NotImplementedError


class OAuthDialogRedirectView(View, OAuthMixin):
    """
    Base view that handles the redirection to the service
    authorization dialog.

    """
    def post(self, request, *args, **kwargs):
        # Clears the current session to avoid conflicts.
        self.session_clear()

        # Fetches the url where the user will be redirected when the
        # flow ends and stores it in the user's session.
        if 'next' in request.POST:
            self.session_put(**{'next': request.POST['next']})

        else:
            self.session_put(**{'next': '/'})

        if 'only_login' in request.POST:
            self.session_put(**{'only_login': request.POST['only_login']})

        if self.client.oauth_version == 1:
            # Gets the OAuth request token.
            credentials = self.client.get_request_token(
                callback=self.get_callback_url())

            # Appends the response to the user session.
            self.session_put(**{
                'request_token': credentials.get('oauth_token'),
                'request_token_secret': credentials.get('oauth_token_secret')
            })

        # Redirects the user to the authorization dialog.
        return redirect(self.get_redirect_url())


class OAuthCallbackView(View, OAuthMixin):
    """
    Base view that handles the callback redirection from the service.

    """
    def get(self, request, *args, **kwargs):
        # Protects the view to be accessed by non OAuth requests.
        if self.client.verifier_label not in request.GET:
            return HttpResponseForbidden()

        # Gets the OAuth access token.
        if self.client.oauth_version == 1:
            credentials = self.client.get_access_token(
                request_token=self.session_get('request_token'),
                request_token_secret=self.session_get('request_token_secret'),
                verifier=request.GET[self.client.verifier_label]
            )
        elif self.client.oauth_version == 2:
            credentials = self.client.get_access_token(
                verifier=request.GET[self.client.verifier_label],
                callback=self.get_callback_url(),
            )

        # Verifies if the service's user id was provided in the credentials,
        # if it is not provided the fetches is by debugging the access token.
        access_token = credentials.get(self.client.access_token_label, None)
        service_uid = credentials.get(self.client.uid_label, None)
        token_expiration = credentials.get(self.client.expiration_label, None)

        # Calculates the token expiration date.
        if token_expiration:
            token_expiration = (
                timezone.now() + timedelta(seconds=int(token_expiration)))

        # Debugs the token.
        if not service_uid:
            token_is_valid, debugged_data = self.client.debug_access_token(
                access_token)
            if token_is_valid:
                service_uid = debugged_data.get(self.client.uid_label)

        # Appends the response to the user session.
        self.session_put(**{
            'access_token': credentials.get(
                self.client.access_token_label),
            'access_token_secret': credentials.get(
                self.client.access_token_secret_label),
            'service_uid': service_uid
        })

        # Gets the lookups to retrieve or create the profile.
        lookup_kwargs = {
            'service_uid': self.session_get('service_uid')
        }

        # Tries to retrieve the profile.
        try:
            profile = self.client.model.objects.get(**lookup_kwargs)
            created = False

        except self.client.model.DoesNotExist:
            profile = None

        # If there is no profile and the button clicked was only login
        # redirects the user to the page where it comes from, otherwise
        # creates a new profile with the given data.
        if not profile:
            if self.session_get('only_login'):
                parsed = urlparse(self.session_pop('only_login'))
                params = parse_qs(parsed.query)
                params.update(oauth_error=True)

                return redirect(self.client.encode_url(parsed.path, params))

            else:
                profile = self.client.model.objects.create(**lookup_kwargs)
                created = True

        # Updates the profile to make sure that we have always the most
        # recent token.
        profile.oauth_access_token = self.session_get('access_token')

        if self.client.oauth_version == 1:
            profile.oauth_request_token = self.session_get('request_token')
            profile.oauth_request_token_secret = self.session_get(
                'request_token_secret')
            profile.oauth_access_token_secret = self.session_get(
                'access_token_secret')

        if self.client.oauth_version == 2:
            profile.oauth_access_token_expires_at = (
                token_expiration if token_expiration else None)

        profile.save()

        # If the profile was created or has no user, tries to attach the
        # profile to current user, if the user is not logged in redirects
        # it to the final setup view to create a new one.
        if created or not profile.user:
            if request.user.is_authenticated():
                profile.user = self.request.user
                profile.save()

                # Tells to the site that the user has connected its profile.
                connect.send(
                    sender=self, user=self.request.user,
                    service=self.client.service_name.lower()
                )

                # Tells to the user that his connection was successful.
                messages.success(request, _(
                    'Your %(service)s profile was successfully connected '
                    'with your user account.'
                ) % {'service': self.client.service_name})

            else:
                self.session_put(**{'new_user': True})

        elif (profile.user and profile.user != self.request.user and
                self.request.user.is_authenticated()):

            # Tells to the user that his connection was unsuccessful.
            messages.error(request, _(
                'This %(service)s profile is already connected with another '
                'user account.'
            ) % {'service': self.client.service_name})

        else:
            # Logs the user in if it is not logged yet.
            self.client.login(request, self.session_get('service_uid'))

            # Tells to the site that the user was logged in.
            login.send(
                sender=self, user=self.request.user,
                service=self.client.service_name.lower()
            )

        return redirect(self.get_redirect_url())


class OAuthSetupView(TemplateView, OAuthMixin):
    """
    Base View that handles the setup for the account after the access
    token is provided.

    """
    template_name = SETUP_TEMPLATE

    def get_profile(self):
        # Tries to return the OAuth profile by retrieving the service_uid
        # in the current session.
        try:
            return self.client.model.objects.get(
                service_uid=self.session_get('service_uid'))

        except self.client.model.DoesNotExist:
            return None

    def create_new_user(self, data):
        # Creates the new user.
        user = User.objects.create_user(
            data['username'], email=data['email']
        )

        # Updates the user data.
        user.first_name = data['first_name']
        user.last_name = data['last_name']
        user.save()

        return user

    def get_context_data(self, **kwargs):
        context = super(OAuthSetupView, self).get_context_data(**kwargs)
        context['service'] = self.client.service_name
        context['form'] = kwargs.get('form', SETUP_FORM_CLASS(kwargs))
        return context

    def get(self, request, *args, **kwargs):
        # Protects the view to be accessed by non OAuth requests.
        if (request.user.is_authenticated() or
                not self.session_get('service_uid')):

            return HttpResponseForbidden()

        # Fetches the current OAuth profile.
        profile = self.get_profile()

        # If the profile has no user tries to attach the current user to it,
        # if the user is not logged in then creates a new one.
        if not profile.user:
            # Fetches the user data from the service.
            user_data = self.retrieve_user_data()

            if EMAIL_IS_USERNAME and 'email' in user_data:
                user_data.update({'username': user_data['email']})

            # If the data is incomplete then redirects the user to the final
            # setup form.
            if ('username' not in user_data or 'email' not in user_data or
                    [(k, v) for k, v in user_data.items() if k in
                        ['username', 'email'] and not v]):

                return self.render_to_response(
                    self.get_context_data(**user_data))

            else:
                # Tries to fetch an exiting user with that matches the data
                # retrieved from the service.
                try:
                    username_filter = Q(username=user_data['username'])
                    email_filter = Q(email=user_data['email'])
                    user = User.objects.get(username_filter | email_filter)

                # If no user is found, then creates a new user with the data
                # retrieved from the service.
                except User.DoesNotExist:
                    user = self.create_new_user(user_data)

                # Attaches the user to the profile.
                profile.user = user
                profile.save()

                # Tells to the site that the user has connected its profile.
                connect.send(
                    sender=self, user=user,
                    service=self.client.service_name.lower()
                )

                # Authenticates the new user.
                self.client.login(request, self.session_get('service_uid'))

                # Tells to the site that the user was logged in.
                login.send(
                    sender=self, user=user,
                    service=self.client.service_name.lower()
                )

                # Redirects the user to the proper url.
                return redirect(self.get_redirect_url())

    def post(self, request, *args, **kwargs):
        # Protects the view to be accessed by non OAuth requests.
        if (request.user.is_authenticated() or
                not self.session_get('service_uid')):

            return HttpResponseForbidden()

        # Fetches the current OAuth profile.
        profile = self.get_profile()

        form = SETUP_FORM_CLASS(request.POST)
        if form.is_valid():
            # Composes the data for the user creation.
            user_data = {
                'email': form.cleaned_data['email'],
                'first_name': form.cleaned_data['first_name'],
                'last_name': form.cleaned_data['last_name']
            }

            # Sets the user's username.
            if EMAIL_IS_USERNAME:
                user_data.update({'username': user_data['email']})
            else:
                user_data.update({'username': form.cleaned_data['username']})

            # Gets a new user and attaches it to the profile.
            user = self.create_new_user(user_data)
            profile.user = user
            profile.save()

            # Tells to the site that the user has connected its profile.
            connect.send(
                sender=self, user=user,
                service=self.client.service_name.lower()
            )

            # Authenticates the new user.
            self.client.login(request, self.session_get('service_uid'))

            # Tells to the site that the user was logged in.
            login.send(
                sender=self, user=user,
                service=self.client.service_name.lower()
            )

            # Redirects the user to the proper url.
            return redirect(self.get_redirect_url())

        else:
            return self.render_to_response(self.get_context_data(
                **{'form': form}))


class OAuthDisconnectView(View, OAuthMixin):
    def get_profile(self):
        # Tries to return the OAuth profile by retrieving the current
        # user's id.
        try:
            return self.client.model.objects.get(
                user__id=self.request.user.id)

        except self.client.model.DoesNotExist:
            return None

    def post(self, request, *args, **kwargs):
        # Fetches the OAuth profile.
        profile = self.get_profile()

        # If the profile is given then deletes it.
        if profile:
            user = profile.user
            profile.delete()

            # Tells to the site that the profile was disconnected.
            disconnect.send(
                sender=self, user=user,
                service=self.client.service_name.lower()
            )

            # Tells to the user that the disconnection was successful.
            messages.success(request, _(
                'Your %(service)s profile was successfully disconnected from '
                'your user account.'
            ) % {'service': self.client.service_name})

        return redirect(request.POST.get('next', '/'))



import uuid

from django.core.urlresolvers import reverse

from socialnetworks.base.views import (
    OAuthDialogRedirectView, OAuthCallbackView,
    OAuthSetupView, OAuthDisconnectView
)
from socialnetworks.github.clients import GitHubClient


class GitHubDialogRedirect(OAuthDialogRedirectView):
    client = GitHubClient()

    def get_callback_url(self):
        return self.client.get_domain() + reverse(
            'socialnetworks:github:callback')

    def get_redirect_url(self):
        return self.client.encode_url(self.client.authorization_url, {
            'client_id': self.client.app_key,
            'redirect_uri': self.get_callback_url(),
            'scope': self.client.scope,
            'state': uuid.uuid4()
        })


class GitHubCallback(OAuthCallbackView):
    client = GitHubClient()

    def get_callback_url(self):
        return self.client.get_domain() + reverse(
            'socialnetworks:github:callback')

    def get_redirect_url(self):
        if self.session_get('new_user'):
            return reverse('socialnetworks:github:setup')

        else:
            return self.session_pop('next') or '/'


class GitHubSetup(OAuthSetupView):
    client = GitHubClient()

    def get_redirect_url(self):
        return self.session_pop('next') or '/'

    def retrieve_user_data(self):
        # Retrieves the proper profile.
        profile = self.get_profile()

        # Creates a client that can make signed requests.
        gh = GitHubClient(profile)

        # Defines the fields to retrieve.
        #fields = ','.join(['first_name', 'last_name', 'email', 'username'])
        data = gh.get('user')

        # Parses the "name" of GitHub, simply splits the name by white
        # spaces, the first name is the first element of the resulting list,
        # the last name are the remaining elements joined again by white
        # spaces. If there are not white spaces in the name, then the first
        # name is the name returned by GitHub and the last name is
        # left blak.
        name = data['name']
        first_name = name.split(' ')[0] if ' ' in name else name
        last_name = ' '.join(name.split(' ')[1:]) if ' ' in name else None

        return {
            'first_name': first_name,
            'last_name': last_name,
            'email': data['email']
        }


class GitHubOAuthDisconnect(OAuthDisconnectView):
    client = GitHubClient()



from socialnetworks.base.backends import BaseSocialBackend
from socialnetworks.facebook.models import FacebookOAuthProfile


class FacebookBackend(BaseSocialBackend):
    model = FacebookOAuthProfile



import uuid

from django.core.urlresolvers import reverse

from socialnetworks.base.views import (
    OAuthDialogRedirectView, OAuthCallbackView,
    OAuthSetupView, OAuthDisconnectView
)
from socialnetworks.linkedin.clients import LinkedInClient


class LinkedInDialogRedirect(OAuthDialogRedirectView):
    client = LinkedInClient()

    def get_callback_url(self):
        return self.client.get_domain() + reverse(
            'socialnetworks:linkedin:callback')

    def get_redirect_url(self):
        return self.client.encode_url(self.client.authorization_url, {
            'client_id': self.client.app_key,
            'redirect_uri': self.get_callback_url(),
            'response_type': 'code',
            'scope': self.client.scope,
            'state': uuid.uuid4(),
        })


class LinkedInCallback(OAuthCallbackView):
    client = LinkedInClient()

    def get_callback_url(self):
        return self.client.get_domain() + reverse(
            'socialnetworks:linkedin:callback')

    def get_redirect_url(self):
        if self.session_get('new_user'):
            return reverse('socialnetworks:linkedin:setup')

        else:
            return self.session_pop('next') or '/'


class LinkedInSetup(OAuthSetupView):
    client = LinkedInClient()

    def get_redirect_url(self):
        return self.session_pop('next') or '/'

    def retrieve_user_data(self):
        # Retrieves the proper profile.
        profile = self.get_profile()

        # Creates a client that can make signed requests.
        li = LinkedInClient(profile)

        # Defines the fields to retrieve.
        fields = ','.join(['firstName', 'lastName', 'email-address'])

        # Fetches the user's data from LinkedIn.
        r = li.get('people/~:(%s)' % fields)

        return {
            'first_name': r['firstName'],
            'last_name': r['lastName'],
            'email': r['emailAddress'],
        }


class LinkedInOAuthDisconnect(OAuthDisconnectView):
    client = LinkedInClient()



from socialnetworks.base.clients import OAuth2Client
from socialnetworks.linkedin import settings
from socialnetworks.linkedin.models import LinkedInOAuthProfile


class LinkedInClient(OAuth2Client):
    # Configuration
    service_name = 'LinkedIn'
    app_key = settings.APP_ID
    app_secret = settings.APP_SECRET
    model = LinkedInOAuthProfile

    uid_label = 'id'
    expiration_label = 'expires_in'

    service_api_url = 'https://api.linkedin.com/v1/'
    authorization_url = 'https://www.linkedin.com/uas/oauth2/authorization'
    access_token_url = 'https://www.linkedin.com/uas/oauth2/accessToken'
    token_debug_url = 'people/~:(id)'
    session_key = 'socialnetworks:linkedin'

    def get_auth_params(self, token=None):
        # Pass the access token as a parameter for the LinkedIn OAuth2
        # service and tell to return the response in json format.
        if self.profile and not token:
            token = self.profile.oauth_access_token

        return {'format': 'json', 'oauth2_access_token': token}

    def get(self, api_endpoint, params={}, token=None):
        params.update(self.get_auth_params(token))
        url = self.service_api_url + api_endpoint

        return self.__request_get__(url, params=params).json()

    def post(self, api_endpoint, data=None, params={}, token=None):
        params.update(self.get_auth_params(token))
        url = self.service_api_url + api_endpoint

        return self.__request_post__(url, data=data, params=params).json()

    def debug_access_token(self, token=None):
        if self.profile and not token:
            token = self.profile.oauth_access_token

        r = self.get(self.token_debug_url, token=token)

        return (False if 'errorCode' in r else True, r)



from django.core.exceptions import ImproperlyConfigured

from socialnetworks.base.settings import CONFIGURATION, COOKIE_MAX_AGE


# Tries to get the Twitter configuration, if the configuration is not
# provided or is incorrect raises an ImproperlyConfigured exception.
TWITTER = CONFIGURATION.get('TWITTER', None)

if not TWITTER:
    raise ImproperlyConfigured(
        'You have socialnetworks.facebook in your ISTALLED_APPS, but you do '
        'not specify any "TWITTER" settings inside '
        'SOCIALNETWORK_CONFIGURATION.'
    )

else:
    APP_ID = TWITTER.get('APP_ID', None)
    APP_SECRET = TWITTER.get('APP_SECRET', None)
    SESSION_KEY = TWITTER.get('SESSION_KEY', 'dsntw')
    SESSION_FIELDS = TWITTER.get('SESSION_FIELDS', ['screen_name', 'name'])

    if not APP_ID:
        raise ImproperlyConfigured(
            'A proper "APP_ID" must be specified in order to use '
            'socialnetworks.facebook module.'
        )

    if not APP_SECRET:
        raise ImproperlyConfigured(
            'A proper "APP_SECRET" must be specified in order to use '
            'socialnetworks.facebook module.'
        )



from socialnetworks.base.utils import read_social_data
from socialnetworks.facebook.settings import SESSION_KEY


def read_facebook_data(request):
    """
    Returns the current user's Facebook data if it was previously fetched
    and stored in the user's session or cookies, otherwise returns None.

    """
    return read_social_data(request, SESSION_KEY)



from django.core.urlresolvers import reverse

from socialnetworks.base.views import (
    OAuthDialogRedirectView, OAuthCallbackView,
    OAuthSetupView, OAuthDisconnectView
)
from socialnetworks.twitter.clients import TwitterClient


class TwitterDialogRedirect(OAuthDialogRedirectView):
    client = TwitterClient()

    def get_callback_url(self):
        return self.client.get_domain() + reverse(
            'socialnetworks:twitter:callback')

    def get_redirect_url(self):
        return self.client.encode_url(self.client.authorization_url, {
            'oauth_token': self.session_get('request_token')})


class TwitterCallback(OAuthCallbackView):
    client = TwitterClient()

    def get_redirect_url(self):
        if self.session_get('new_user'):
            return reverse('socialnetworks:twitter:setup')

        else:
            return self.session_pop('next') or '/'


class TwitterSetupView(OAuthSetupView):
    client = TwitterClient()

    def get_redirect_url(self):
        return self.session_pop('next') or '/'

    def retrieve_user_data(self):
        # Retrieves the proper profile.
        profile = self.get_profile()

        # Creates a client that can make signed requests.
        tw = TwitterClient(profile)

        # Fetches the user's data from Twitter.
        data = tw.get(self.client.token_debug_url)

        # Parses the "name" of Twitter, simply splits the name by white
        # spaces, the first name is the first element of the resulting list,
        # the last name are the remaining elements joined again by white
        # spaces. If there are not white spaces in the name, then the first
        # name is the name returned by Twitter and the last name is
        # left blak.
        name = data['name']
        first_name = name.split(' ')[0] if ' ' in name else name
        last_name = ' '.join(name.split(' ')[1:]) if ' ' in name else None

        return {
            'first_name': first_name,
            'last_name': last_name,
        }


class TwitterOAuthDisconnect(OAuthDisconnectView):
    client = TwitterClient()



from socialnetworks.base.clients import OAuth2Client
from socialnetworks.github import settings
from socialnetworks.github.models import GitHubOAuthProfile


class GitHubClient(OAuth2Client):
    # Configuration
    service_name = 'GitHub'
    app_key = settings.APP_ID
    app_secret = settings.APP_SECRET
    app_access_token = settings.APP_ACCESS_TOKEN
    scope = settings.SCOPE
    model = GitHubOAuthProfile

    uid_label = 'id'
    expiration_label = 'expires'

    authorization_url = 'https://github.com/login/oauth/authorize'
    access_token_url = 'https://github.com/login/oauth/access_token'
    token_debug_url = 'user'
    service_api_url = 'https://api.github.com/'
    session_key = 'socialnetworks:github'

    def debug_access_token(self, token=None):
        if self.profile and not token:
            token = self.profile.oauth_access_token

        r = self.get(self.token_debug_url, auth_params={'access_token': token})

        return (False if 'message' in r else True, r)



from socialnetworks.base.clients import OAuth1Client
from socialnetworks.twitter import settings
from socialnetworks.twitter.models import TwitterOAuthProfile


class TwitterClient(OAuth1Client):
    # Configuration
    service_name = 'Twitter'
    app_key = settings.APP_ID
    app_secret = settings.APP_SECRET
    model = TwitterOAuthProfile

    service_api_url = 'https://api.twitter.com/1.1/'
    request_token_url = 'https://api.twitter.com/oauth/request_token'
    authorization_url = 'https://api.twitter.com/oauth/authorize'
    access_token_url = 'https://api.twitter.com/oauth/access_token'
    token_debug_url = 'account/verify_credentials.json'
    session_key = 'socialnetworks:twitter'

    def debug_access_token(self):
        if self.profile:
            r = self.get(self.token_debug_url)

            if 'errors' in r:
                return (False, r)
            else:
                return (True, r)



from socialnetworks.base.backends import BaseSocialBackend
from socialnetworks.linkedin.models import LinkedInOAuthProfile


class LinkedInBackend(BaseSocialBackend):
    model = LinkedInOAuthProfile



from django.core.urlresolvers import reverse

from socialnetworks.base.views import (
    OAuthDialogRedirectView, OAuthCallbackView,
    OAuthSetupView, OAuthDisconnectView
)
from socialnetworks.facebook.clients import FacebookClient


class FacebookDialogRedirect(OAuthDialogRedirectView):
    client = FacebookClient()

    def get_callback_url(self):
        return self.client.get_domain() + reverse(
            'socialnetworks:facebook:callback')

    def get_redirect_url(self):
        return self.client.encode_url(self.client.authorization_url, {
            'client_id': self.client.app_key,
            'redirect_uri': self.get_callback_url(),
            'scope': self.client.scope,
        })


class FacebookCallback(OAuthCallbackView):
    client = FacebookClient()

    def get_callback_url(self):
        return self.client.get_domain() + reverse(
            'socialnetworks:facebook:callback')

    def get_redirect_url(self):
        if self.session_get('new_user'):
            return reverse('socialnetworks:facebook:setup')

        else:
            return self.session_pop('next') or '/'


class FacebookSetup(OAuthSetupView):
    client = FacebookClient()

    def get_redirect_url(self):
        return self.session_pop('next') or '/'

    def retrieve_user_data(self):
        # Retrieves the proper profile.
        profile = self.get_profile()

        # Creates a client that can make signed requests.
        fb = FacebookClient(profile)

        # Defines the fields to retrieve.
        fields = ','.join(['first_name', 'last_name', 'email', 'username'])

        # Fetches the user's data from Facebook.
        return fb.get('me', params={'fields': fields})


class FacebookOAuthDisconnect(OAuthDisconnectView):
    client = FacebookClient()



from django.core import signing

from socialnetworks.base.settings import COOKIE_MAX_AGE


def read_social_data(request, key):
    """
    Search the current request for user's social information, search first
    in the user's session, if there is no information then search in the
    request's cookies.

    Returns a dictionary of the social information if found, otherwise
    returns None.

    """
    data = request.session.get(key, request.get_signed_cookie(
        key, max_age=COOKIE_MAX_AGE, default=None))

    return signing.loads(data) if data else None



from uuid import uuid4

from django.core.urlresolvers import reverse

from .clients import PayPalClient
from ..base import views


class PayPalDialogRedirect(views.OAuthDialogRedirectView):
    client = PayPalClient()

    def get_callback_url(self):
        return self.request.build_absolute_uri(reverse(
            'socialnetworks:paypal:callback'))

    def get_redirect_url(self):
        return self.client.encode_url(self.client.authorization_url, {
            'client_id': self.client.app_key,
            'redirect_uri': self.get_callback_url(),
            'response_type': 'code',
            'scope': self.client.scope,
            'nonce': uuid4()
        })


class PayPalCallback(views.OAuthCallbackView):
    client = PayPalClient()

    def get_callback_url(self):
        return self.request.build_absolute_uri(reverse(
            'socialnetworks:paypal:callback'))

    def get_redirect_url(self):
        if self.session_get('new_user'):
            return reverse('socialnetworks:paypal:setup')

        else:
            return self.session_pop('next') or '/'


class PayPalSetup(views.OAuthSetupView):
    client = PayPalClient()

    def get_redirect_url(self):
        return self.session_pop('next') or '/'

    def retrieve_user_data(self):
        # Retrieves the proper profile.
        profile = self.get_profile()

        # Creates a client that can make signed requests.
        pp = PayPalClient(profile)

        # Retrieve the user info from the api.
        data = pp.get(pp.token_debug_url)

        return {
            'first_name': data['given_name'],
            'last_name': data['family_name'],
            'email': data['email']
        }


class PayPalOAuthDisconnect(views.OAuthDisconnectView):
    client = PayPalClient()



from django.utils.translation import ugettext_lazy as _

from socialnetworks.base.models import BaseOAuth2Profile


class GitHubOAuthProfile(BaseOAuth2Profile):
    class Meta:
        verbose_name = _('github profile')
        verbose_name_plural = _('github profiles')



from socialnetworks.base.clients import OAuth2Client
from socialnetworks.facebook import settings
from socialnetworks.facebook.models import FacebookOAuthProfile


class FacebookClient(OAuth2Client):
    # Configuration
    service_name = 'Facebook'
    app_key = settings.APP_ID
    app_secret = settings.APP_SECRET
    app_access_token = settings.APP_ACCESS_TOKEN
    scope = settings.SCOPE
    model = FacebookOAuthProfile

    expiration_label = 'expires'

    authorization_url = 'https://www.facebook.com/dialog/oauth'
    access_token_url = 'https://graph.facebook.com/oauth/access_token'
    token_debug_url = 'debug_token'
    service_api_url = 'https://graph.facebook.com/'
    session_key = 'socialnetworks:facebook'

    # Returns the app access token if it is defined in settings,
    # otherwise fetches the token from Facebook.
    def get_app_access_token(self):
        if self.app_access_token:
            return self.app_access_token

        else:
            params = {
                'client_id': self.app_key,
                'client_secret': self.app_secret,
                'grant_type': 'client_credentials'
            }
            r = self.__request_get__(self.access_token_url, params=params)

            return self.parse_response(r.content)['access_token']

    def debug_access_token(self, token=None):
        if self.profile and not token:
            token = self.profile.oauth_access_token

        r = self.get(
            self.token_debug_url, params={'input_token': token},
            auth_params={'access_token': self.get_app_access_token()}
        )

        if 'data' in r and r['data']['is_valid']:
            return (True, r['data'])

        else:
            return (False, None)
